# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# Copyright 2010-2011 OpenStack Foundation.
# Copyright 2012 Justin Santa Barbara
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_db import exception as oslo_db_exception
from oslo_db.sqlalchemy import utils
from oslo_log import log
import sqlalchemy
from sqlalchemy import exc as sqlalchemy_exc
from sqlalchemy import select


from designate import exceptions
from designate.i18n import _
from designate.storage import sql

LOG = log.getLogger(__name__)

RRSET_FILTERING_INDEX = {
    'created_at': 'recordset_created_at',
    'updated_at': 'rrset_updated_at',
    'zone_id': 'rrset_zoneid',
    'name': 'recordset_type_name',
    'type': 'rrset_type',
    'ttl': 'rrset_ttl',
    'tenant_id': 'rrset_tenant_id',
}


# copy from olso/db/sqlalchemy/utils.py
def paginate_query(query, table, limit, sort_keys, marker=None,
                   sort_dir=None, sort_dirs=None):

    # Add sorting
    query, sort_dirs = sort_query(query, table, sort_keys, sort_dir=sort_dir)

    # Add pagination
    if marker is not None:
        marker_values = []
        for sort_key in sort_keys:
            v = getattr(marker, sort_key)
            marker_values.append(v)

        # Build up an array of sort criteria as in the docstring
        criteria_list = []
        for i in range(len(sort_keys)):
            crit_attrs = []
            for j in range(i):
                table_attr = getattr(table.c, sort_keys[j])
                crit_attrs.append(table_attr == marker_values[j])

            table_attr = getattr(table.c, sort_keys[i])
            if sort_dirs[i] == 'desc':
                crit_attrs.append(table_attr < marker_values[i])
            else:
                crit_attrs.append(table_attr > marker_values[i])

            criteria = sqlalchemy.sql.and_(*crit_attrs)
            criteria_list.append(criteria)

        f = sqlalchemy.sql.or_(*criteria_list)
        query = query.where(f)

    if limit is not None:
        query = query.limit(limit)

    return query


def sort_query(query, table, sort_keys, sort_dir=None, sort_dirs=None):

    if 'id' not in sort_keys:
        # TODO(justinsb): If this ever gives a false-positive, check
        # the actual primary key, rather than assuming its id
        LOG.warning('Id not in sort_keys; is sort_keys unique?')

    assert (not (sort_dir and sort_dirs))

    # Default the sort direction to ascending
    if sort_dirs is None and sort_dir is None:
        sort_dir = 'asc'

    # Ensure a per-column sort direction
    if sort_dirs is None:
        sort_dirs = [sort_dir for _sort_key in sort_keys]

    assert (len(sort_dirs) == len(sort_keys))

    for current_sort_key, current_sort_dir in zip(sort_keys, sort_dirs):
        try:
            sort_dir_func = {
                'asc': sqlalchemy.asc,
                'desc': sqlalchemy.desc,
            }[current_sort_dir]
        except KeyError:
            raise ValueError(_("Unknown sort direction, "
                               "must be 'desc' or 'asc'"))
        try:
            sort_key_attr = getattr(table.c, current_sort_key)
        except AttributeError:
            raise utils.InvalidSortKey()
        query = query.order_by(sort_dir_func(sort_key_attr))

    return query, sort_dirs


def check_marker(table, marker):

    marker_query = select(table).where(table.c.id == marker)

    try:
        with sql.get_read_session() as session:
            marker_resultproxy = session.execute(marker_query)
            marker = marker_resultproxy.fetchone()
        if marker is None:
            raise exceptions.MarkerNotFound(
                'Marker %s could not be found' % marker)
    except oslo_db_exception.DBError as e:
        # Malformed UUIDs return StatementError wrapped in a
        # DBError
        if isinstance(e.inner_exception,
                      sqlalchemy_exc.StatementError):
            raise exceptions.InvalidMarker()
        else:
            raise

    return marker


def get_rrset_index(sort_key):
    rrset_index_hint = None
    index = RRSET_FILTERING_INDEX.get(sort_key)

    if index:
        rrset_index_hint = 'USE INDEX (%s)' % index

    return rrset_index_hint



# Copyright 2014 Hewlett-Packard Development Company, L.P.
#
# Author: Kiall Mac Innes <kiall@hpe.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_log import log as logging

import designate.conf
from designate.conf.mdns import DEFAULT_MDNS_PORT
from designate import dnsmiddleware
from designate import dnsutils
from designate.mdns import handler
from designate import service
from designate import storage
from designate import utils


CONF = designate.conf.CONF
LOG = logging.getLogger(__name__)


class Service(service.Service):
    _dns_default_port = DEFAULT_MDNS_PORT

    def __init__(self):
        self._storage = None

        super().__init__(
            self.service_name, threads=CONF['service:mdns'].threads,
        )
        self.dns_service = service.DNSService(
            self.dns_application, self.tg,
            CONF['service:mdns'].listen,
            CONF['service:mdns'].tcp_backlog,
            CONF['service:mdns'].tcp_keepidle,
            CONF['service:mdns'].tcp_recv_timeout,
        )

    def start(self):
        super().start()
        self.dns_service.start()

    def stop(self, graceful=True):
        self.dns_service.stop()
        super().stop(graceful)

    @property
    def storage(self):
        if not self._storage:
            self._storage = storage.get_storage()
        return self._storage

    @property
    def service_name(self):
        return 'mdns'

    @property
    @utils.cache_result
    def dns_application(self):
        # Create an instance of the RequestHandler class and wrap with
        # necessary middleware.
        application = handler.RequestHandler(self.storage, self.tg)
        application = dnsmiddleware.TsigInfoMiddleware(
            application, self.storage
        )
        application = dnsmiddleware.SerializationMiddleware(
            application, dnsutils.TsigKeyring(self.storage)
        )

        return application



# Copyright 2020 Cloudification GmbH. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_log import log as logging
import pecan

from designate.api.v2.controllers import rest
from designate.common import keystone
from designate.objects.adapters import DesignateAdapter
from designate.objects import SharedZone
from designate import utils

LOG = logging.getLogger(__name__)


class SharedZonesController(rest.RestController):
    SORT_KEYS = ['created_at', 'updated_at', ]

    @pecan.expose(template='json:', content_type='application/json')
    @utils.validate_uuid('zone_id', 'zone_share_id')
    def get_one(self, zone_id, zone_share_id):
        """Get Zone Share"""
        request = pecan.request
        context = request.environ['context']

        zone = self.central_api.get_shared_zone(
                context, zone_id, zone_share_id)

        LOG.info(
            "Retrieved %(zone)s",
            {"zone": zone}
        )

        return DesignateAdapter.render('API_v2', zone, request=request)

    @pecan.expose(template='json:', content_type='application/json')
    @utils.validate_uuid('zone_id')
    def get_all(self, zone_id, **params):
        """List all Shared Zones"""
        request = pecan.request
        context = request.environ['context']

        # Extract the pagination params
        marker, limit, sort_key, sort_dir = utils.get_paging_params(
            context, params, self.SORT_KEYS)

        # Extract any filter params
        accepted_filters = ('target_project_id',)
        criterion = self._apply_filter_params(
            params, accepted_filters, {})

        criterion['zone_id'] = zone_id

        shared_zones = self.central_api.find_shared_zones(
            context, criterion, marker, limit, sort_key, sort_dir)

        LOG.info("Retrieved %(shared_zones)s", {'shared_zones': shared_zones})

        return DesignateAdapter.render('API_v2', shared_zones, request=request)

    @pecan.expose(template='json:', content_type='application/json')
    @utils.validate_uuid('zone_id')
    def post_all(self, zone_id):
        """Share Zone"""
        request = pecan.request
        response = pecan.response
        context = request.environ['context']

        payload = request.body_dict

        keystone.verify_project_id(
            context, payload.get('target_project_id', None)
        )

        zone_share = DesignateAdapter.parse('API_v2', payload, SharedZone())

        zone_share = self.central_api.share_zone(context, zone_id, zone_share)

        response.status_int = 201

        LOG.info(
            "Shared %(shared_zone)s",
            {'shared_zone': zone_share}
        )

        return DesignateAdapter.render(
            'API_v2', zone_share, request=request)

    @pecan.expose(template='json:', content_type='application/json')
    @utils.validate_uuid('zone_id', 'zone_share_id')
    def delete_one(self, zone_id, zone_share_id):
        """Unshare Zone"""
        request = pecan.request
        response = pecan.response
        context = request.environ['context']

        zone = self.central_api.unshare_zone(context, zone_id, zone_share_id)
        response.status_int = 204

        LOG.info("Unshared %(zone)s", {'zone': zone})



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_log import log as logging
import oslotest.base

from designate.common import constants
from designate import objects

LOG = logging.getLogger(__name__)


class QuotaTest(oslotest.base.BaseTestCase):
    def test_quota_min(self):
        for current_quota in constants.VALID_QUOTAS:
            quota = objects.Quota(tenant_id='123', resource=current_quota,
                                  hard_limit=constants.MIN_QUOTA)

            self.assertEqual('123', quota.tenant_id)
            self.assertEqual(current_quota, quota.resource)
            self.assertEqual(constants.MIN_QUOTA, quota.hard_limit)

    def test_quota_max(self):
        for current_quota in constants.VALID_QUOTAS:
            quota = objects.Quota(tenant_id='123', resource=current_quota,
                                  hard_limit=constants.MAX_QUOTA)

            self.assertEqual('123', quota.tenant_id)
            self.assertEqual(current_quota, quota.resource)
            self.assertEqual(constants.MAX_QUOTA, quota.hard_limit)

    def test_quota_too_small(self):
        for current_quota in constants.VALID_QUOTAS:
            self.assertRaises(ValueError, objects.Quota, tenant_id='123',
                              resource=current_quota,
                              hard_limit=constants.MIN_QUOTA - 1)

    def test_quota_too_large(self):
        for current_quota in constants.VALID_QUOTAS:
            self.assertRaises(ValueError, objects.Quota, tenant_id='123',
                              resource=current_quota,
                              hard_limit=constants.MAX_QUOTA + 1)

    def test_quota_invalid(self):
        for current_quota in constants.VALID_QUOTAS:
            self.assertRaises(ValueError, objects.Quota, tenant_id='123',
                              resource=current_quota,
                              hard_limit='bogus')

    def test_quota_list(self):
        quotas = objects.QuotaList()
        quotas.append(objects.Quota(
            tenant_id='123', resource=constants.QUOTA_RECORDSET_RECORDS))
        quotas.append(objects.Quota(tenant_id='123',
                                    resource=constants.QUOTA_ZONE_RECORDS))
        quotas.append(objects.Quota(tenant_id='123',
                                    resource=constants.QUOTA_ZONE_RECORDSETS))

        self.assertEqual(constants.QUOTA_RECORDSET_RECORDS, quotas[0].resource)
        self.assertEqual(constants.QUOTA_ZONE_RECORDS, quotas[1].resource)
        self.assertEqual(constants.QUOTA_ZONE_RECORDSETS, quotas[2].resource)

    def test_quota_list_from_dict(self):
        quotas = objects.QuotaList().from_dict({
            constants.QUOTA_ZONES: 100,
            constants.QUOTA_ZONE_RECORDSETS: 101,
            constants.QUOTA_ZONE_RECORDS: 102,
            constants.QUOTA_RECORDSET_RECORDS: 103,
            constants.QUOTA_API_EXPORT_SIZE: 104,
        })

        self.assertEqual(constants.QUOTA_ZONES, quotas[0].resource)
        self.assertEqual(100, quotas[0].hard_limit)
        self.assertEqual(constants.QUOTA_API_EXPORT_SIZE, quotas[4].resource)
        self.assertEqual(104, quotas[4].hard_limit)

    def test_quota_list_to_dict(self):
        quotas = objects.QuotaList().from_dict({
            constants.QUOTA_ZONES: 100,
            constants.QUOTA_ZONE_RECORDSETS: 101,
        })

        self.assertEqual(100, quotas.to_dict()[constants.QUOTA_ZONES])
        self.assertEqual(101,
                         quotas.to_dict()[constants.QUOTA_ZONE_RECORDSETS])



# Copyright 2013 Hewlett-Packard Development Company, L.P.
#
# Author: Kiall Mac Innes <kiall@managedit.ie>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_log import log as logging

from designate.tests.functional.api import v2

LOG = logging.getLogger(__name__)


class ApiV2RecordSetsTest(v2.ApiV2TestCase):
    def setUp(self):
        super().setUp()
        self.zone = self.create_zone()

    def test_get_recordsets_owned_by_project(self):
        response = self.client.get(
            '/recordsets', headers={'X-Test-Role': 'member'}
        )

        self.assertIn('recordsets', response.json)
        self.assertIn('links', response.json)
        self.assertIn('self', response.json['links'])

        self.assertEqual(2, len(response.json['recordsets']))

        fixture = self.get_recordset_fixture(self.zone['name'], fixture=0)
        response = self.client.post_json(
            '/zones/%s/recordsets' % self.zone['id'], fixture,
            headers={'X-Test-Role': 'member'}
        )

        # Check the headers are what we expect
        self.assertEqual(201, response.status_int)
        self.assertEqual('application/json', response.content_type)

        response = self.client.get(
            '/recordsets', headers={'X-Test-Role': 'member'}
        )

        self.assertIn('recordsets', response.json)
        self.assertIn('links', response.json)
        self.assertIn('self', response.json['links'])

        self.assertEqual(3, len(response.json['recordsets']))

    def test_get_recordset_redirects(self):
        fixture = self.get_recordset_fixture(self.zone['name'], fixture=0)
        response = self.client.post_json(
            '/zones/%s/recordsets' % self.zone['id'], fixture,
            headers={'X-Test-Role': 'member'}
        )

        # Check the headers are what we expect
        self.assertEqual(201, response.status_int)
        self.assertEqual('application/json', response.content_type)

        url = '/recordsets/%s' % response.json['id']
        response = self.client.get(url, headers={'X-Test-Role': 'member'})
        self.assertEqual(301, response.status_int)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.mport threading
from oslo_config import cfg
from oslo_config import fixture as cfg_fixture
import oslotest.base

import designate.conf
from designate.worker.tasks import base

CONF = designate.conf.CONF


class TestTaskConfig(oslotest.base.BaseTestCase):
    def setUp(self):
        super().setUp()
        self.useFixture(cfg_fixture.Config(CONF))
        self.task_config = base.TaskConfig()

    def test_load_config(self):
        self.assertIsInstance(
            self.task_config.config, cfg.ConfigOpts.GroupAttr
        )

    def test_threshold_percentage(self):
        CONF.set_override('threshold_percentage', 51, 'service:worker')

        self.assertIsNone(self.task_config._threshold_percentage)
        self.assertEqual(51, self.task_config.threshold_percentage)
        self.assertIsNotNone(self.task_config._threshold_percentage)
        self.assertEqual(51, self.task_config.threshold_percentage)

    def test_timeout(self):
        CONF.set_override('poll_timeout', 52, 'service:worker')

        self.assertIsNone(self.task_config._timeout)
        self.assertEqual(52, self.task_config.timeout)
        self.assertIsNotNone(self.task_config._timeout)
        self.assertEqual(52, self.task_config.timeout)

    def test_retry_interval(self):
        CONF.set_override('poll_retry_interval', 53, 'service:worker')
        self.assertEqual(53, self.task_config.retry_interval)

    def test_max_retries(self):
        CONF.set_override('poll_max_retries', 54, 'service:worker')
        self.assertEqual(54, self.task_config.max_retries)

    def test_delay(self):
        CONF.set_override('poll_delay', 55, 'service:worker')
        self.assertEqual(55, self.task_config.delay)

    def test_max_prop_time(self):
        CONF.set_override('threshold_percentage', 100, 'service:worker')
        CONF.set_override('poll_timeout', 10, 'service:worker')
        CONF.set_override('poll_retry_interval', 10, 'service:worker')
        CONF.set_override('poll_max_retries', 10, 'service:worker')
        CONF.set_override('poll_delay', 3, 'service:worker')

        self.assertIsNone(self.task_config._max_prop_time)
        self.assertEqual(203, self.task_config.max_prop_time)
        self.assertIsNotNone(self.task_config._max_prop_time)
        self.assertEqual(203, self.task_config.max_prop_time)



# Copyright 2016 Hewlett Packard Enterprise Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import abc

from oslo_log import log as logging

LOG = logging.getLogger(__name__)


class Filter(metaclass=abc.ABCMeta):
    """This is the base class used for filtering Pools.

    This class should implement a single public function
    :func:`filter` which accepts
    a :class:`designate.objects.pool.PoolList` and returns a
    :class:`designate.objects.pool.PoolList`
    """

    name = ''

    def __init__(self, storage):
        self.storage = storage
        LOG.debug('Loaded %s filter in chain' % self.name)

    @abc.abstractmethod
    def filter(self, context, pools, zone):
        """Filter list of supplied pools based on attributes in the request

        :param context: :class:`designate.context.DesignateContext` - Context
            Object from request
        :param pools: :class:`designate.objects.pool.PoolList` - List of pools
            to choose from
        :param zone: :class:`designate.objects.zone.Zone` - Zone to be created
        :return: :class:`designate.objects.pool.PoolList` - Filtered list of
            Pools
        """



# COPYRIGHT 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_log import log as logging
import pecan

from designate.api.v2.controllers import rest
from designate import policy
from designate import utils

LOG = logging.getLogger(__name__)


class ExportController(rest.RestController):

    @pecan.expose(template=None, content_type='text/dns')
    @utils.validate_uuid('zone_id')
    def get_one(self, zone_id):
        context = pecan.request.environ['context']
        policy.check('zone_export', context)

        zones = self.central_api.export_zone(context, zone_id)

        return zones



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_log import log as logging
import oslotest.base

from designate import exceptions
from designate import objects

LOG = logging.getLogger(__name__)


class RRDataAAAATest(oslotest.base.BaseTestCase):
    def test_valid_aaaa_record(self):
        recordset = objects.RecordSet(
            name='www.example.test.', type='AAAA',
            records=objects.RecordList(objects=[
                objects.Record(data='2001:db8:0:1::1'),
            ])
        )
        recordset.validate()

    def test_reject_a_record(self):
        recordset = objects.RecordSet(
            name='www.example.test.', type='AAAA',
            records=objects.RecordList(objects=[
                objects.Record(data='192.0.2.1'),
            ])
        )
        self.assertRaisesRegex(
            exceptions.InvalidObject,
            'Provided object does not match schema',
            recordset.validate
        )

    def test_reject_invalid_data(self):
        recordset = objects.RecordSet(
            name='www.example.test.', type='AAAA',
            records=objects.RecordList(objects=[
                objects.Record(data='TXT'),
            ])
        )
        self.assertRaisesRegex(
            exceptions.InvalidObject,
            'Provided object does not match schema',
            recordset.validate
        )



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from oslo_config import cfg

from designate.conf import api
from designate.conf import base  # noqa
from designate.conf import central
from designate.conf import coordination
from designate.conf import dynect
from designate.conf import heartbeat_emitter
from designate.conf import keystone
from designate.conf import mdns
from designate.conf import network_api
from designate.conf import producer
from designate.conf import proxy
from designate.conf import sink
from designate.conf import storage
from designate.conf import worker

CONF = cfg.CONF

base.register_opts(CONF)
api.register_opts(CONF)
central.register_opts(CONF)
coordination.register_opts(CONF)
dynect.register_opts(CONF)
heartbeat_emitter.register_opts(CONF)
keystone.register_opts(CONF)
mdns.register_opts(CONF)
network_api.register_opts(CONF)
producer.register_opts(CONF)
proxy.register_opts(CONF)
sink.register_opts(CONF)
storage.register_opts(CONF)
worker.register_opts(CONF)



# Copyright 2013 Hewlett-Packard Development Company, L.P.
#
# Author: Kiall Mac Innes <kiall@hpe.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_log import log as logging

from designate.quota import base

LOG = logging.getLogger(__name__)


class NoopQuota(base.Quota):
    __plugin_name__ = 'noop'

    def _get_quotas(self, context, tenant_id):
        """Internal Get Quotas used by get_quotas"""
        return {}

    def get_quota(self, context, tenant_id, resource):
        """Get Quota"""

    def set_quota(self, context, tenant_id, resource, hard_limit):
        """Set Quota"""

    def reset_quotas(self, context, tenant_id):
        """Reset Quotas"""



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from oslo_log import versionutils
from oslo_policy import policy

from designate.common import constants
from designate.common.policies import base


DEPRECATED_REASON = """
The shared zones API now supports system scope and default roles.
"""

deprecated_get_shared_zone = policy.DeprecatedRule(
    name="get_zone_share",
    check_str=base.RULE_ADMIN_OR_OWNER,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)

deprecated_share_zone = policy.DeprecatedRule(
    name="share_zone",
    check_str=base.RULE_ADMIN_OR_OWNER,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)

deprecated_find_project_zone_share = policy.DeprecatedRule(
    name="find_project_zone_share",
    check_str=base.RULE_ADMIN_OR_OWNER,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)

deprecated_unshare_zone = policy.DeprecatedRule(
    name="unshare_zone",
    check_str=base.RULE_ADMIN_OR_OWNER,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)

rules = [
    policy.DocumentedRuleDefault(
        name="get_zone_share",
        check_str=base.SYSTEM_ADMIN_OR_PROJECT_MEMBER,
        scope_types=[constants.PROJECT],
        description="Get a Zone Share",
        operations=[
            {
                'path': '/v2/zones/{zone_id}/shares/{zone_share_id}',
                'method': 'GET'
            }
        ],
        deprecated_rule=deprecated_get_shared_zone
    ),
    policy.DocumentedRuleDefault(
        name="share_zone",
        check_str=base.SYSTEM_ADMIN_OR_PROJECT_MEMBER,
        scope_types=[constants.PROJECT],
        description="Share a Zone",
        operations=[
            {
                'path': '/v2/zones/{zone_id}/shares',
                'method': 'POST'
            }
        ],
        deprecated_rule=deprecated_share_zone
    ),
    policy.DocumentedRuleDefault(
        name="find_zone_shares",
        # Using rule ANY here because the search criteria will narrow the
        # results appropriate for the API call.
        check_str=base.RULE_ANY,
        description="List Shared Zones",
        operations=[
            {
                'path': '/v2/zones/{zone_id}/shares',
                'method': 'GET'
            }
        ]
    ),
    policy.RuleDefault(
        name="find_project_zone_share",
        check_str=base.SYSTEM_ADMIN_OR_PROJECT_MEMBER,
        scope_types=[constants.PROJECT],
        description="Check the can query for a specific projects shares.",
        deprecated_rule=deprecated_find_project_zone_share
    ),
    policy.DocumentedRuleDefault(
        name="unshare_zone",
        check_str=base.SYSTEM_ADMIN_OR_PROJECT_MEMBER,
        scope_types=[constants.PROJECT],
        description="Unshare Zone",
        operations=[
            {
                'path': '/v2/zones/{zone_id}/shares/{shared_zone_id}',
                'method': 'DELETE'
            }
        ],
        deprecated_rule=deprecated_unshare_zone
    )
]


def list_rules():
    return rules



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from unittest import mock

from oslo_log import log as logging

from designate.manage import base
from designate.manage import pool
from designate import objects
from designate.tests import base_fixtures
import designate.tests.functional


LOG = logging.getLogger(__name__)


def hydrate_pool_targets(target_masters):
    pool_targets = objects.PoolTargetList()
    masters = objects.PoolTargetMasterList()
    for target_master in target_masters:
        masters.append(target_master)
    target = objects.PoolTarget(masters=masters)
    target.masters = masters
    pool_targets.append(target)
    return pool_targets


class UpdatePoolTestCase(designate.tests.functional.TestCase):
    def setUp(self):
        super().setUp()
        self.stdlog = base_fixtures.StandardLogging()
        self.useFixture(self.stdlog)

        self.print_result = mock.patch.object(
            base.Commands, '_print_result').start()

    def test_update_pools_zones(self):
        values = dict(
            name='example.com.',
            email='info@example.com',
            type='PRIMARY'
        )

        zone = self.central_service.create_zone(
            self.admin_context, zone=objects.Zone.from_dict(values))

        # Ensure the correct NS Records are in place
        new_pool = self.central_service.get_pool(
            self.admin_context, zone.pool_id
        )

        new_pool.targets = hydrate_pool_targets([objects.PoolTargetMaster(
            pool_target_id=new_pool.id,
            host='192.0.2.2',
            port='53')]
        )

        command = pool.PoolCommands()
        command._setup()

        with mock.patch.object(
                self.central_service, 'update_zone') as mock_update_zone:
            command._update_zones(new_pool)
            mock_update_zone.assert_called_once()

    def test_update_pools_zones_multiple_masters(self):
        values = dict(
            name='example.com.',
            email='info@example.com',
            type='PRIMARY'
        )

        zone = self.central_service.create_zone(
            self.admin_context, zone=objects.Zone.from_dict(values))

        # Ensure the correct NS Records are in place
        new_pool = self.central_service.get_pool(
            self.admin_context, zone.pool_id
        )

        targets1 = hydrate_pool_targets([
            objects.PoolTargetMaster(
                pool_target_id=new_pool.id,
                host='192.0.2.3',
                port='53')
        ])
        targets2 = hydrate_pool_targets([
            objects.PoolTargetMaster(
                pool_target_id=new_pool.id,
                host='192.0.2.4',
                port='53')
        ])
        new_pool.targets = objects.PoolTargetList()
        new_pool.targets.extend(targets1.objects + targets2.objects)

        command = pool.PoolCommands()
        command._setup()

        command._update_zones(new_pool)



# Copyright 2015 Hewlett-Packard Development Company, L.P.
#
# Author: Endre Karlson <endre.karlson@hpe.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import shutil
import tempfile
from unittest import mock

from oslo_config import fixture as cfg_fixture
import oslotest.base
import tooz.coordination

import designate.conf
from designate import coordination
from designate.tests import base_fixtures


CONF = designate.conf.CONF


class TestCoordination(oslotest.base.BaseTestCase):
    def setUp(self):
        super().setUp()
        self.useFixture(cfg_fixture.Config(CONF))
        self.service_name = 'service_name'
        self.tg = mock.Mock()
        self.tempdir = tempfile.mkdtemp()
        CONF.set_override('backend_url', "file://%s" % self.tempdir,
                          group='coordination')
        self.addCleanup(shutil.rmtree, self.tempdir, ignore_errors=True)

    def test_retry_if_tooz_error(self):
        self.assertFalse(coordination._retry_if_tooz_error(Exception()))
        self.assertTrue(coordination._retry_if_tooz_error(
            tooz.coordination.ToozError('error'))
        )

    def test_start(self):
        service = coordination.Coordination(self.service_name, self.tg)
        service.start()
        self.assertTrue(service.started)
        service.stop()

    def test_start_with_grouping_enabled(self):
        service = coordination.Coordination(
            self.service_name, self.tg, grouping_enabled=True
        )
        service.start()
        self.assertTrue(service.started)
        self.assertIn(service.name,
                      service.coordinator.get_groups().get())
        self.assertIn(service.coordination_id,
                      service.coordinator.get_members(service.name).get())
        service.stop()

    def test_stop(self):
        service = coordination.Coordination(self.service_name, self.tg)
        service.start()
        service.stop()
        self.assertFalse(service.started)

    def test_stop_with_grouping_enabled(self):
        service = coordination.Coordination(
            self.service_name, self.tg, grouping_enabled=True
        )
        service.start()
        service.stop()
        self.assertFalse(service.started)

    def test_start_no_coordination(self):
        CONF.set_override('backend_url', None, group='coordination')
        # self.config(backend_url=None, group="coordination")
        service = coordination.Coordination(self.service_name, self.tg)
        service.start()
        self.assertIsNone(service.coordinator)

    def test_stop_no_coordination(self):
        CONF.set_override('backend_url', None, group='coordination')
        service = coordination.Coordination(self.service_name, self.tg)
        self.assertIsNone(service.coordinator)
        service.start()
        service.stop()

    def test_get_lock(self):
        service = coordination.Coordination(self.service_name, self.tg)
        service._coordinator = mock.Mock()

        service.get_lock(b'lock')

        service._coordinator.get_lock.assert_called_with(b'lock')

    def test_un_watchers(self):
        service = coordination.Coordination(self.service_name, self.tg)
        service._started = True
        service._coordinator = mock.Mock()

        service._coordinator_run_watchers()

        service._coordinator.run_watchers.assert_called_with()

    def test_run_watchers_not_started(self):
        service = coordination.Coordination(self.service_name, self.tg)
        service._started = False
        service._coordinator = mock.Mock()

        service._coordinator_run_watchers()

        service._coordinator.run_watchers.assert_not_called()

    def test_create_group(self):
        service = coordination.Coordination(self.service_name, self.tg)
        service._coordinator = mock.Mock()

        service._create_group()

        service._coordinator.create_group.assert_called_with(b'service_name')

    def test_create_group_already_exists(self):
        service = coordination.Coordination(self.service_name, self.tg)
        service._coordinator = mock.Mock()
        service._coordinator.create_group.side_effect = (
            tooz.coordination.GroupAlreadyExist('')
        )

        service._create_group()

        service._coordinator.create_group.assert_called_with(b'service_name')

    def test_disable_grouping(self):
        service = coordination.Coordination(self.service_name, self.tg)
        service._coordinator = mock.Mock()

        service._disable_grouping()

        service._coordinator.leave_group.assert_called_with(b'service_name')

    def test_disable_grouping_already_exists(self):
        service = coordination.Coordination(self.service_name, self.tg)
        service._coordinator = mock.Mock()
        service._coordinator.leave_group.side_effect = (
            tooz.coordination.GroupNotCreated('')
        )

        service._disable_grouping()

        service._coordinator.leave_group.assert_called_with(b'service_name')


@mock.patch('tenacity.nap.time', mock.Mock())
class TestPartitioner(oslotest.base.BaseTestCase):
    def setUp(self):
        super().setUp()
        self.tempdir = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, self.tempdir, ignore_errors=True)

    def _get_partitioner(self, partitions, host=b'a'):
        fixture = self.useFixture(base_fixtures.CoordinatorFixture(
            "file://%s" % self.tempdir, host)
        )
        group = 'group'
        fixture.coordinator.create_group(group)
        fixture.coordinator.join_group(group)

        return coordination.Partitioner(fixture.coordinator, group, host,
                                        partitions), fixture.coordinator

    def test_callbacks(self):
        cb1 = mock.Mock()
        cb2 = mock.Mock()
        partitions = list(range(0, 10))

        p_one, c_one = self._get_partitioner(partitions)
        p_one.start()
        p_one.watch_partition_change(cb1)
        p_one.watch_partition_change(cb2)

        # Initial partitions are calucated upon service bootup
        cb1.assert_called_with(partitions, None, None)
        cb2.assert_called_with(partitions, None, None)

        cb1.reset_mock()
        cb2.reset_mock()

        # Startup a new partioner that will cause the cb's to be called
        p_two, c_two = self._get_partitioner(partitions, host=b'b')
        p_two.start()

        # We'll get the 5 first partition ranges
        c_one.run_watchers()
        cb1.assert_called_with(partitions[:5], [b'a', b'b'], mock.ANY)
        cb2.assert_called_with(partitions[:5], [b'a', b'b'], mock.ANY)

    def test_two_even_partitions(self):
        partitions = list(range(0, 10))

        p_one, c_one = self._get_partitioner(partitions)
        p_two, c_two = self._get_partitioner(partitions, host=b'b')

        p_one.start()
        p_two.start()

        # Call c_one watchers making it refresh it's partitions
        c_one.run_watchers()

        self.assertEqual([0, 1, 2, 3, 4], p_one.my_partitions)
        self.assertEqual([5, 6, 7, 8, 9], p_two.my_partitions)

    def test_two_odd_partitions(self):
        partitions = list(range(0, 11))

        p_one, c_one = self._get_partitioner(partitions)
        p_two, c_two = self._get_partitioner(partitions, host=b'b')

        p_one.start()
        p_two.start()

        # Call c_one watchers making it refresh it's partitions
        c_one.run_watchers()

        self.assertEqual([0, 1, 2, 3, 4, 5], p_one.my_partitions)
        self.assertEqual([6, 7, 8, 9, 10], p_two.my_partitions)

    def test_three_even_partitions(self):
        partitions = list(range(0, 10))

        p_one, c_one = self._get_partitioner(partitions)
        p_two, c_two = self._get_partitioner(partitions, host=b'b')
        p_three, c_three = self._get_partitioner(partitions, host=b'c')

        p_one.start()
        p_two.start()
        p_three.start()

        # Call c_one watchers making it refresh it's partitions
        c_one.run_watchers()
        c_two.run_watchers()

        self.assertEqual([0, 1, 2, 3], p_one.my_partitions)
        self.assertEqual([4, 5, 6, 7], p_two.my_partitions)
        self.assertEqual([8, 9], p_three.my_partitions)

    def test_three_odd_partitions(self):
        partitions = list(range(0, 11))

        p_one, c_one = self._get_partitioner(partitions)
        p_two, c_two = self._get_partitioner(partitions, host=b'b')
        p_three, c_three = self._get_partitioner(partitions, host=b'c')

        p_one.start()
        p_two.start()
        p_three.start()

        c_one.run_watchers()
        c_two.run_watchers()

        self.assertEqual([0, 1, 2, 3], p_one.my_partitions)
        self.assertEqual([4, 5, 6, 7], p_two.my_partitions)
        self.assertEqual([8, 9, 10], p_three.my_partitions)

    def test_get_members(self):
        partitioner = coordination.Partitioner(
            mock.Mock(), 'group', 'host', mock.Mock()
        )

        mock_get_members = mock.Mock()

        partitioner._coordinator.get_members = mock.Mock()
        partitioner._coordinator.get_members.return_value = mock_get_members

        partitioner._get_members('group_id')

        mock_get_members.get.assert_called_with()

    def test_get_members_group_not_created(self):
        partitioner = coordination.Partitioner(
            mock.Mock(), 'group', 'host', mock.Mock()
        )

        mock_get_members = mock.Mock()

        partitioner._coordinator.get_members = mock.Mock()
        partitioner._coordinator.get_members.return_value = mock_get_members

        mock_get_members.get.side_effect = tooz.coordination.GroupNotCreated(
            'group_id'
        )

        self.assertRaisesRegex(
            tooz.coordination.GroupNotCreated,
            'Group group_id does not exist',
            partitioner._get_members, 'group_id'
        )

    def test_get_members_tooz_error(self):
        partitioner = coordination.Partitioner(
            mock.Mock(), 'group', 'host', mock.Mock()
        )

        mock_get_members = mock.Mock()

        partitioner._coordinator.get_members = mock.Mock()
        partitioner._coordinator.get_members.return_value = mock_get_members

        mock_get_members.get.side_effect = tooz.coordination.ToozError(
            'error'
        )

        self.assertRaisesRegex(
            tooz.coordination.ToozError,
            'error',
            partitioner._get_members, 'group_id'
        )

    def test_unwatch_partition_change(self):
        partitioner = coordination.Partitioner(
            mock.Mock(), 'group', 'host', mock.Mock()
        )
        partitioner._callbacks = mock.Mock()

        partitioner.unwatch_partition_change('partition')

        partitioner._callbacks.remove.assert_called_with('partition')


class TestPartitionerWithoutBackend(oslotest.base.BaseTestCase):
    def test_start(self):
        # We test starting the partitioner and calling the watch func first
        partitions = list(range(0, 10))

        cb1 = mock.Mock()
        cb2 = mock.Mock()
        partitioner = coordination.Partitioner(
            None, 'group', 'meme', partitions)
        partitioner.watch_partition_change(cb1)
        partitioner.watch_partition_change(cb2)
        partitioner.start()
        cb1.assert_called_with(partitions, None, None)
        cb2.assert_called_with(partitions, None, None)

    def test_cb_on_watch(self):
        partitions = list(range(0, 10))
        cb = mock.Mock()

        partitioner = coordination.Partitioner(
            None, 'group', 'meme', partitions)
        partitioner.start()
        partitioner.watch_partition_change(cb)
        cb.assert_called_with(partitions, None, None)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from unittest import mock

import dns
import dns.query
import dns.tsigkeyring

import designate.conf
from designate import dnsmiddleware
from designate import dnsutils
from designate.mdns import handler
from designate import storage
import designate.tests.functional


CONF = designate.conf.CONF


class TestSerializationMiddleware(designate.tests.functional.TestCase):
    def setUp(self):
        super().setUp()
        self.storage = storage.get_storage()
        self.tg = mock.Mock()

    def test_with_tsigkeyring(self):
        self.create_tsigkey(fixture=1)

        query = dns.message.make_query(
            'example.com.', dns.rdatatype.SOA,
        )
        query.use_tsig(dns.tsigkeyring.from_text(
            {'test-key-two': 'AnotherSecretKey'})
        )
        payload = query.to_wire()

        application = handler.RequestHandler(self.storage, self.tg)
        application = dnsmiddleware.SerializationMiddleware(
            application, dnsutils.TsigKeyring(self.storage)
        )

        self.assertTrue(next(application(
            {'payload': payload, 'addr': ['192.0.2.1', 5353]}
        )))

    def test_without_tsigkeyring(self):
        query = dns.message.make_query(
            'example.com.', dns.rdatatype.SOA,
        )
        payload = query.to_wire()

        application = handler.RequestHandler(self.storage, self.tg)
        application = dnsmiddleware.SerializationMiddleware(
            application, dnsutils.TsigKeyring(self.storage)
        )

        self.assertTrue(next(application(
            {'payload': payload, 'addr': ['192.0.2.1', 5353]}
        )))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from oslo_config import fixture as cfg_fixture
import oslotest.base

import designate.conf
from designate import context
from designate.network_api import fake
from designate.network_api import get_network_api


CONF = designate.conf.CONF


class FakeNetworkAPITest(oslotest.base.BaseTestCase):
    def setUp(self):
        super().setUp()
        self.useFixture(cfg_fixture.Config(CONF))

        self.api = get_network_api('fake')
        self.context = context.DesignateContext(
            user_id='12345', project_id='54321',
        )
        self.addCleanup(fake.reset_floatingips)

    def test_list_floatingips(self):
        fake.allocate_floatingip(self.context.project_id)
        fake.allocate_floatingip('12345')

        self.assertEqual(1, len(self.api.list_floatingips(self.context)))

    def test_list_floatingips_is_admin(self):
        fake.allocate_floatingip(self.context.project_id)
        fake.allocate_floatingip('12345')
        self.context.is_admin = True

        self.assertEqual(2, len(self.api.list_floatingips(self.context)))

    def test_allocate_floatingip(self):
        fip = fake.allocate_floatingip(self.context.project_id)

        self.assertIn('192.0.2', fip['address'])
        self.assertEqual('RegionOne', fip['region'])
        self.assertIn('id', fip)

    def test_deallocate_floatingip(self):
        fip = fake.allocate_floatingip(self.context.project_id)

        self.assertIn('192.0.2', fip['address'])
        self.assertEqual('RegionOne', fip['region'])
        self.assertIn('id', fip)

        self.assertIsNone(fake.deallocate_floatingip(self.context.project_id))
